import postcss from 'postcss';

/**
 * Check if a selector should be whitelisted and / or replaced.
 */
function filterSelector(_ref) {
  var ruleSelector = _ref.ruleSelector,
    ruleParentSelectors = _ref.ruleParentSelectors,
    filters = _ref.filters;
  var newSelector = "";
  filters.some(function (selectorFilter) {
    var selector = selectorFilter.selector || selectorFilter;
    var replacementSelector = selectorFilter.replacement;
    var parentComparisonSelector = replacementSelector || selector;
    var selectorsAreEqual = selector === ruleSelector;
    var parentSelectorIsEqual = ruleParentSelectors.reduce(function (bool, ruleParentSelector) {
      var result = parentComparisonSelector instanceof RegExp ? parentComparisonSelector.test(ruleParentSelector) : ruleParentSelector === parentComparisonSelector;
      return result;
    }, false);
    var selectorsMatch = selector instanceof RegExp && selector.test(ruleSelector);
    if (selectorsAreEqual || parentSelectorIsEqual || selectorsMatch) {
      newSelector = replacementSelector ? ruleSelector.replace(selector, replacementSelector) : ruleSelector;

      // Do not stop iterating over the selector filters if the parent selector was matched
      // because child selectors may get replaced in a further iteration.
      if (!parentSelectorIsEqual) {
        return true;
      }
    }
    return false;
  });
  return newSelector;
}

var PRESERVE_LINES_START = "START preserve lines";
var PRESERVE_LINES_END = "preserve lines END";

/**
 * Provide a PostCSS plugin for extracting and replacing CSS selectors.
 */
function postcssSelectorExtract() {
  var filters = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var preserveLines = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  return postcss.plugin("postcss-extract-selectors", function () {
    return function (nodes) {
      // We have to force `any` type, because postcss type
      // definitions seem to be outdated.
      nodes.walkRules(function (rule) {
        var ruleSelectors = rule.selector.split(",").map(function (ruleSelector) {
          return ruleSelector.replace(/(\r\n|\n|\r)/gm, "").trim();
        }).map(function (ruleSelector) {
          return filterSelector({
            ruleSelector: ruleSelector,
            ruleParentSelectors: rule.parent.selector ? rule.parent.selector.split(",") : [],
            filters: filters
          });
        }).filter(function (ruleSelector) {
          return ruleSelector.length;
        });
        if (ruleSelectors.length) {
          // eslint-disable-next-line no-param-reassign
          rule.selector = ruleSelectors.join(",");
        } else {
          if (preserveLines) {
            var ruleLines = rule.toString().split(/\r\n|\r|\n/).length;
            rule.cloneBefore({
              type: "comment",
              text: "".concat(PRESERVE_LINES_START).concat("\n".repeat(ruleLines - 1)).concat(PRESERVE_LINES_END),
              raws: Object.assign(rule.raws, {
                left: " ",
                right: " "
              })
            });
          }
          rule.remove();
        }
      });

      // Remove empty @ rules.
      nodes.walkAtRules(function (rule) {
        if (rule.nodes && !rule.nodes.length) {
          rule.remove();
        }
      });
    };
  });
}

/**
 * Synchronously extract and replace CSS selectors from a string.
 */
var processSync = function processSync(_ref) {
  var css = _ref.css,
    filters = _ref.filters,
    postcssSyntax = _ref.postcssSyntax,
    preserveLines = _ref.preserveLines;
  return postcss(postcssSelectorExtract(filters, preserveLines)).process(css, {
    syntax: postcssSyntax
  }).css.replace(new RegExp("\\/\\* ".concat(PRESERVE_LINES_START, "|").concat(PRESERVE_LINES_END, " \\*\\/"), "g"), "");
};

/**
 * Asynchronously extract and replace CSS selectors from a string.
 */
var process = function process(options) {
  return new Promise(function (resolve) {
    var result = processSync(options);
    resolve(result);
  });
};

export { processSync, process };
